from bokeh.io import vform, hplot
from bokeh.plotting import figure, output_file, show
from bokeh.models import CustomJS, ColumnDataSource, HoverTool
from bokeh.models.widgets import Dropdown, CheckboxButtonGroup

from collections import OrderedDict
import pandas as pd
import numpy as np

output_file("bokeh_olympics.html")

# Read the data into a pands dataframe
df = pd.read_csv('olympics_full.csv')

countries = df.Country.unique()
all_years = map(str, df.Year.unique())

# Compute the count of each type of medeal for each country by year
medal_types = ["Gold", "Silver", "Bronze"]

medal_counts_by_country = {'Gold':{}, 'Silver':{}, 'Bronze':{}}

for country in countries:
    for medal in medal_types:
        medal_counts_by_country[medal][country] = df.loc[df['Country'] == country]\
                                          .loc[df['Medal'] == medal]\
                                          .groupby('Year')\
                                          .count()['Medal']

# Compute the count of all medals for each country by year
total_counts_by_country_year = {}

for country in countries:
    total_counts_by_country_year[country] = {}

    for year in map(int, all_years):
        total_counts_by_country_year[country][year] = df.loc[df['Country'] == country]\
                                                          .loc[df['Year'] == year]\
                                                          .count()['Medal']

# Only Use Countries that have more than X medals
min_metals = 5

x_countries = []

for country in countries:
    if (np.sum(medal_counts_by_country["Gold"][country])
        + np.sum(medal_counts_by_country["Silver"][country])
        + np.sum(medal_counts_by_country["Bronze"][country])) > min_metals:
        x_countries.append(country)
        
selected_year = 2002
heights = {"Gold":[], "Silver":[], "Bronze":[]}

for country in x_countries:
    for medal in medal_types:
        curr_years = map(int, medal_counts_by_country[medal][country].keys())

        try:
            year_idx = curr_years.index(selected_year)
            heights[medal].append(medal_counts_by_country[medal][country].values[year_idx])
        except ValueError:
            heights[medal].append(0)
       
y_vals = {"Gold" : [h/2.0 for h in heights["Gold"]], 
          "Silver" : np.add([h/2.0 for h in heights["Silver"]], heights["Gold"]),
          "Bronze" : np.add(np.add([h/2.0 for h in heights["Bronze"]], heights["Silver"]), heights["Gold"])}            

gold_source = ColumnDataSource(data=dict(x=x_countries, y=y_vals["Gold"], height=heights["Gold"]))
silver_source = ColumnDataSource(data=dict(x=x_countries, y=y_vals["Silver"], height=heights["Silver"]))
bronze_source = ColumnDataSource(data=dict(x=x_countries, y=y_vals["Bronze"], height=heights["Bronze"]))

years = [[], [], []]
counts = [[], [], []]

for country in x_countries:
    for medal_idx, medal in enumerate(medal_types):
        years[medal_idx].append(map(int, medal_counts_by_country[medal][country].keys()))
        counts[medal_idx].append(medal_counts_by_country[medal][country].values)
    
x_countries_dup = [x_countries, x_countries, x_countries]    
    
mbc = ColumnDataSource(data=dict(countries=x_countries_dup, years=years, counts=counts))

p_bars = figure(x_range=x_countries, y_range=[0,40], plot_width=800, plot_height=600,
          toolbar_location="left", tools="reset,save,pan,box_zoom,hover")

p_bars.rect(x='x', y='y', height='height', source=gold_source, width = 0.5, color="Yellow")
p_bars.rect(x='x', y='y', height='height', source=silver_source, width = 0.5, color="SlateGray")
p_bars.rect(x='x', y='y', height='height', source=bronze_source, width = 0.5, color="Sienna")


hm_years = []
hm_countries = []
hm_counts = []
hm_colors = []
hm_selected = []

max_count = max([max(total_counts_by_country_year[x].values()) for x in total_counts_by_country_year])

for country in x_countries:
    for year in map(int, all_years):
        hm_countries.append(country)
        hm_years.append(str(year))
        hm_counts.append(total_counts_by_country_year[country][year])

        if year == selected_year:
            hm_selected.append(0);
            color = "rgba(255, 11, 0, {})".format(float(hm_counts[-1]) / float(max_count))
        else:
            hm_selected.append(1);
            color = "rgba(255, 215, 0, {})".format(float(hm_counts[-1]) / float(max_count))

        hm_colors.append(color)

hm_source = ColumnDataSource(data=dict(x=hm_countries, y=hm_years, 
                                        color=hm_colors, count=hm_counts, selected=hm_selected))

p_hm = figure(x_range=p_bars.x_range, y_range=all_years, plot_width=800, plot_height=600,
          toolbar_location="left", tools="reset,save,pan,box_zoom,hover")

p_hm.rect(x='x', y='y', color='color', source=hm_source, width=0.9, height=0.9)

dd_callback = CustomJS(args=dict(gold_source=gold_source, 
                              silver_source=silver_source,
                              bronze_source=bronze_source,
                              hm_source=hm_source,
                              mbc=mbc), code="""
        var gold_data = gold_source.get('data');
        var silver_data = silver_source.get('data');
        var bronze_data = bronze_source.get('data');

        var hm_data = hm_source.get('data');
        var hm_years = hm_data['y'];
        var hm_colors = hm_data['color'];
        var hm_selected = hm_data['selected'];

        var countries = gold_data['x'];

        y_gold = gold_data['y'];
        y_silver = silver_data['y'];
        y_bronze = bronze_data['y'];
        ys = [y_gold, y_silver, y_bronze];

        h_gold = gold_data['height'];
        h_silver = silver_data['height'];
        h_bronze = bronze_data['height'];
        hs = [h_gold, h_silver, h_bronze];

        var mbc_data = mbc.get('data');
        
        var year_select = +cb_obj.get('value');
        cb_obj.set("label", year_select);
        
        for (var i = 0; i < hm_years.length; i++) {
            if (hm_years[i] == year_select.toString()) {
                hm_selected[i] = 1;
            }
            else {
                hm_selected[i] = 0;
            }
        }


        for (var i = 0; i < hm_colors.length; i++) {
            if (hm_selected[i] == 1) {
                hm_colors[i] = hm_colors[i].replace("215,", "11,");
            }  
            else {
                hm_colors[i] = hm_colors[i].replace("11,", "215,");
            }
        }

        for (var j = 0; j < 3; j++) {
            for (var i = 0; i < countries.length; i++) {
                var years = mbc_data['years'][j][i];
                var year_idx = years.indexOf(year_select)
                
                if (year_idx != -1) {
                    hs[j][i] = mbc_data['counts'][j][i][year_idx];
                    ys[j][i] = hs[j][i]/2.0;

                    for (k = j - 1; k >= 0; k--) {
                        ys[j][i] = ys[j][i] + hs[k][i];
                    }

                }
                else {
                    hs[j][i] = 0;
                    ys[j][i] = 0;
                }
            }
        }
        
        gold_source.trigger('change');
        silver_source.trigger('change');
        bronze_source.trigger('change');
        hm_source.trigger('change');
""")


menu = zip(all_years, all_years)
dropdown = Dropdown(label=str(selected_year), type="warning", menu=menu, callback=dd_callback)

layout = vform(dropdown, hplot(p_bars, p_hm))

p_bars.grid.grid_line_color = None
p_bars.xaxis.major_label_orientation = np.pi/3
p_bars.xaxis.axis_line_color = None

p_hm.xaxis.major_label_orientation = np.pi/3

hm_hover = p_hm.select(dict(type=HoverTool))
hm_hover.tooltips = OrderedDict([
    ('Year', '@y'),
    ('Country', '@x'),
    ('Total Medal Count', '@count')
])

bars_hover = p_bars.select(dict(type=HoverTool))
bars_hover.tooltips = OrderedDict([
    ('Country', '@x'),
    ('Medal Type Count', '@height')
])

show(layout)